//
//  MCPatchGeneralTypes.h
//  mimoComposerCore
//
//  Created by Achim Breidenbach on 02.04.21.
//  Copyright © 2021 Boinx Software International GmbH. All rights reserved.
//

#ifndef MCPatchGeneralTypes_h
#define MCPatchGeneralTypes_h

#include <simd/simd.h>

// Buffer index values shared between shader and C code to ensure Metal shader buffer inputs
// match Metal API buffer set calls.
typedef enum MCPatchGeneralShaderInputIndex
{
	MCPatchGeneralShaderInputIndexVertex = 0,
	MCPatchGeneralShaderInputIndexViewportSize = 1,
	MCPatchGeneralShaderInputIndexUniforms = 2,
	MCPatchGeneralShaderInputIndexProjectionUniforms = 3
} MCPatchGeneralShaderInputIndex;

typedef enum MCPatchLightingFragmentShaderInputIndex
{
	MCPatchFragmentShaderInputIndexLighting = 0
} MCPatchLightingFragmentShaderInputIndex;

struct Uniforms{
	matrix_float4x4 modelViewMatrix;
	matrix_float4x4 normalMatrix;
	simd_float4 color;
};
typedef struct Uniforms Uniforms;

struct LightingUniforms{
	vector_float4 ambientLightColor;
	vector_float4 lightPosition;
	vector_float4 diffuseLightColor;
};
typedef struct LightingUniforms LightingUniforms;

struct ProjectionUniforms{
	matrix_float4x4 projectionMatrix;
};
typedef struct ProjectionUniforms ProjectionUniforms;

// Particle-Emitter related enums and structs
typedef enum MCPatchParticleEmitterShaderInputIndex
{
	MCPatchParticleEmitterShaderInputIndexParticles = 0,
	MCPatchParticleEmitterShaderInputIndexRenderUniforms = 1,
	MCPatchParticleEmitterShaderInputIndexTexture = 0
} MCPatchParticleEmitterShaderInputIndex;

struct ParticleEmitterUniforms{
	matrix_float4x4 modelViewMatrix;
	float timeDeltaMS;
	float startUpDelay;
	float simulationTime;
};
typedef struct ParticleEmitterUniforms ParticleEmitterUniforms;

// Stereoscope-Patch related enums and structs
typedef enum MCPatchStereoscopeShaderInputIndex
{
	MCPatchStereoscopeShaderInputIndexUniforms = 0,
} MCPatchStereoscopeShaderInputIndex;

struct StereoscopeUniforms{
	bool leftEye;
};
typedef struct StereoscopeUniforms StereoscopeUniforms;

struct GradientUniforms {
	vector_float4 color1;
	vector_float4 color2;
};
typedef struct GradientUniforms GradientUniforms;

// Stereoscope-Patch related enums and structs
typedef enum MCPatchGradientShaderInputIndex
{
	MCPatchGradientShaderInputIndexUniforms = 0,
} MCPatchGradientShaderInputIndex;

#endif // MCPatchGeneralTypes_h
